package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.dto.OladingCommonRequest;
import com.qunzhixinxi.hnqz.admin.entity.HuiqiyunInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.LingcaiaiInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.OladingInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.XinbadaInputRecord;
import com.qunzhixinxi.hnqz.admin.service.HuiqiyunInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.LingcaiaiInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.XinbadaInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.gig.GigThirdApiService;
import com.qunzhixinxi.hnqz.admin.service.gig.OladingInputRecordService;
import com.qunzhixinxi.hnqz.admin.util.HMACSignProvider;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * 第三方零工接口
 *
 * @author lixuesong
 * @date 2022年04月17日 11:31
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/gig-third-api")
public class GigThirdApiController {

	private final GigThirdApiService gigThirdApiService;

	private final OladingInputRecordService oladingInputRecordService;

	private final XinbadaInputRecordService xinbadaInputRecordService;

	private final LingcaiaiInputRecordService lingcaiaiInputRecordService;

	private final HuiqiyunInputRecordService huiqiyunInputRecordService;

	private final RedisTemplate redisTemplate;

// 阿拉钉签名验签
//	private static final HMACSignProvider hMACSignProvide = new HMACSignProvider();
//
//	/**
//	 * 公钥
//	 */
//	private static final String PUBLIC_KEY = "123123123";
//
//	/**
//	 * 算法类型
//	 */
//	private static final String ALGORITHM_TYPE = "HmacSHA256";

	/**
	 * 汇企云异步通知回调
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("汇企云异步通知回调")
	@PostMapping("/huiqiyun")
	public String huiQiYunRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("汇企云请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);


		String reqMsgId = bodyJson.getStr("reqMsgId");
		String redisKey = String.format("%s_%s", CacheConstants.HUI_QI_YUN_NOTIFY_KEY, reqMsgId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqMsgId, 30, TimeUnit.MINUTES);
		if (!absent) {
			log.warn("存在在途操作reqMsgId={}", reqMsgId);
			return "FAIL";
		}

		// 请求记录到数据库
		HuiqiyunInputRecord record = new HuiqiyunInputRecord();
		record.setReqMsgId(reqMsgId);
		record.setReqBody(body);
		huiqiyunInputRecordService.save(record);

		gigThirdApiService.huiqiyunNotify(reqMsgId, bodyJson, record.getId());

		return "SUCCESS";
	}

	/**
	 * 浪潮(灵才接好活)结算回调
	 *
	 * @param request
	 * @return
	 */
	@Inner(value = false)
	@SysLog("浪潮异步通知回调")
	@PostMapping("/lingcaiai")
	public String lingcaiaiNotify(HttpServletRequest request) {
		Map<String, String[]> parameterMap = request.getParameterMap();
		log.info("浪潮结算回调请求方法参数:parameterMap={}", JSONUtil.toJsonStr(parameterMap));

		// 签名
		String[] verifies = parameterMap.get("verify");
		String verify = ArrayUtil.isEmpty(verifies) ? "" : verifies[0];
		// 通知类型
		String[] channels = parameterMap.get("channel");
		String channel = ArrayUtil.isEmpty(channels) ? "" : channels[0];
		// 项目单号
		String[] batchNos = parameterMap.get("batch_no");
		String batchNo = ArrayUtil.isEmpty(batchNos) ? "" : batchNos[0];
		// 订单状态
		String[] statuses = parameterMap.get("status");
		String orderStatus = ArrayUtil.isEmpty(statuses) ? "" : statuses[0];
		// project_accounts
		String[] projectAccountsArr = parameterMap.get("project_accounts");
		// 账号单号
//		if (StrUtil.isBlank(projectAccounts)) {
//			log.warn("project_accounts为空");
//			return "fail";
//		}

		String redisKey = String.format("%s_%s_%s", CacheConstants.ZHEGN_QI_ZHI_XING_NOTIFY_KEY, channel, batchNo);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, batchNo, 30, TimeUnit.MINUTES);
		if (!absent) {
			return "success";
		}

		// 请求存入数据库
		LingcaiaiInputRecord record = new LingcaiaiInputRecord();
		record.setVerify(verify);
		record.setChannel(channel);
		record.setReqBody(JSONUtil.toJsonStr(parameterMap));
		record.setBatchNo(batchNo);
		record.setOrderStatus(NumberUtil.isNumber(orderStatus) ? Integer.valueOf(orderStatus) : -1);
		record.setCreateTime(LocalDateTime.now());
		lingcaiaiInputRecordService.save(record);

		return gigThirdApiService.lingcaiaiNotify(channel, batchNo, Integer.valueOf(orderStatus));
	}

	/**
	 * 正启之星异步通知回调
	 *
	 * 接口用于接收批量订单创建结果通知和下发结果通知。
	 * 请注意回调请求头默认的”Content-Type”: “application/octet-stream”
	 * 企业需要给我方响应。通知超时时间10秒，如果没有收到正确响应视为通知失败，失败后重试3次，每次间隔10分钟。
	 *
	 * @param body 请求body
	 * @param headers 请求头
	 */
	@Inner(value = false)
	@SysLog("正启之星异步通知回调")
	@PostMapping("/xinbada")
	public Map<String, Object> xinbadaRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {

		log.info("正启之星请求方法参数:body={}, headers={}", body, headers);

		// 	签名
		String signature = Objects.requireNonNull(headers.get("signature")).get(0);
		// 通知类型（order_check_result_notify：订单校验结果通知，order_call_back_notification：订单下发回调通知，task_redispatch_notification：单笔重试回调通知）
		String actionName = Objects.requireNonNull(headers.get("action_name")).get(0);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);
		Integer orderStatus = bodyJson.getInt("order_status");
		String orderUuid = bodyJson.getStr("order_uuid");
		String orderNo = bodyJson.getStr("order_no");
		String customerUserUuid = bodyJson.getStr("customer_user_uuid");
		String serverUserUuid = bodyJson.getStr("server_user_uuid");

		Map<String, Object> successResult = new HashMap<>();
		if (!StrUtil.isAllNotBlank(signature, actionName, orderNo) || orderStatus == null) {
			log.warn("必传参数为空");
			successResult.put("resp_code", 400);
			successResult.put("resp_message", "必传参数为空");
			successResult.put("success", "false");
			return successResult;
		}

		String redisKey = String.format("%s_%s_%s", CacheConstants.ZHEGN_QI_ZHI_XING_NOTIFY_KEY, actionName, orderNo);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, orderNo, 30, TimeUnit.MINUTES);
		if (!absent) {
			log.warn("存在在途操作orderNo={}", orderNo);
			successResult.put("resp_code", 400);
			successResult.put("resp_message", "存在在途操作");
			successResult.put("success", "false");
			return successResult;
		}

		// 请求记录到数据库
		XinbadaInputRecord xinbadaInputRecord = new XinbadaInputRecord();
		xinbadaInputRecord.setActionName(actionName);
		xinbadaInputRecord.setSignature(signature);
		xinbadaInputRecord.setReqBody(body);
		xinbadaInputRecord.setCreateTime(LocalDateTime.now());
		// body中的部分字段
		xinbadaInputRecord.setOrderStatus(orderStatus);
		xinbadaInputRecord.setOrderUuid(orderUuid);
		xinbadaInputRecord.setOrderNo(orderNo);
		xinbadaInputRecord.setCustomerUserUuid(customerUserUuid);
		xinbadaInputRecord.setServerUserUuid(serverUserUuid);
		xinbadaInputRecordService.save(xinbadaInputRecord);

		gigThirdApiService.xinbadaNotify(actionName, orderUuid, orderStatus);

		successResult.put("resp_code", 200);
		successResult.put("resp_message", "SUCCESS");
		successResult.put("success", "true");

		return successResult;
	}

	/**
	 * 自由职家统一请求地址
	 *
	 * @return
	 */
	@Inner(value = false)
	@SysLog("自由职家统一请求地址")
	@PostMapping("/olading")
	public void oladingRequest(@RequestParam(value = "req", required = false) String reqStr,
							   @RequestParam(required = false) String sign, HttpServletResponse response) {

		log.info("自由职家请求方法参数:req={}, sign={}", reqStr, sign);

		// 请求数据
		OladingCommonRequest.Req req = JSONUtil.toBean(reqStr, OladingCommonRequest.Req.class);
		// 请求接口名称
		String interfaceName = req.getName();
		// 请求data
		Map<String, Object> data = req.getData();
		// 请求requestId
		String requestId = (String) data.get("requestId");
		if (StrUtil.isBlank(requestId)) {
			requestId = (String) data.get("request_id");
		}
		String redisKey = String.format("%s_%s_%s", CacheConstants.OLADING_NOTIFY_KEY, interfaceName, requestId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqStr, 30, TimeUnit.MINUTES);
		if (!absent) {
			log.error("存在在途操作requestId={}", requestId);
			try {
				response.getWriter().write("SUCCESS");
				response.getWriter().flush();
			} catch (Exception e) {
				log.error("response返回异常", e);
			}
			return;
		}

		// 1.请求记录
		OladingInputRecord record = new OladingInputRecord();
		record.setRequestId(requestId);
		record.setInterfaceName(interfaceName);
		record.setReqBody(String.format("%s,%s", reqStr, sign));
		record.setPrivateKey("");
		record.setPublicKey("");
		oladingInputRecordService.save(record);

		try {
			// 2. 验签 TODO 暂不验签
//			boolean verfyResult =
//					hMACSignProvide.verfy(ALGORITHM_TYPE, PUBLIC_KEY, JSONUtil.toJsonStr(req.getData()), sign);
//			log.info("验签结果: {}", verfyResult);

			// 3.实际操作

			if (StrUtil.isNotBlank(requestId)) {
				try {
					// 睡3000ms，解决阿拉钉回调过快，找不到requestId的问题
					Thread.sleep(3000);
				} catch (Exception e) {
					log.warn("sleep异常", e);
				}

				gigThirdApiService.oladingNotify(requestId, req);
			}
		} catch (Exception e) {
			log.error("自由职家统一请求异常", e);
		} finally {
			redisTemplate.delete(redisKey);
			try {
				response.getWriter().write("SUCCESS");
				response.getWriter().flush();
			} catch (Exception e) {
				log.error("response返回异常", e);
			}
		}

	}

}
