/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */
package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReportOpt;
import com.qunzhixinxi.hnqz.admin.entity.WmDaPharmacy;
import com.qunzhixinxi.hnqz.admin.entity.WmDeptTaskTypeConfig;
import com.qunzhixinxi.hnqz.admin.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.entity.dto.WmDeptTaskTypeConfigJsonDTO;
import com.qunzhixinxi.hnqz.admin.entity.vo.WmTaskVO;
import com.qunzhixinxi.hnqz.admin.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.enums.ReportEnum;
import com.qunzhixinxi.hnqz.admin.enums.ScorePackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.TaskStatusEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptRelationMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmReportMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDistributionService;
import com.qunzhixinxi.hnqz.admin.service.WmDaHospitalService;
import com.qunzhixinxi.hnqz.admin.service.WmDaPharmacyService;
import com.qunzhixinxi.hnqz.admin.service.WmDeptTaskTypeConfigService;
import com.qunzhixinxi.hnqz.admin.service.WmReportService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * 报告审核表
 *
 * @author pigx code generator
 * @date 2020-12-21 20:57:26
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmReportServiceImpl extends ServiceImpl<WmReportMapper, WmReport> implements WmReportService {

	private final WmScorePackageService wmScorePackageService;
	private final WmScorePackageStatusService wmScorePackageStatusService;
	private final WmTaskTypeService wmTaskTypeService;
	private final WmTaskService wmTaskService;
	private final WmTaskMapper wmTaskMapper;
	private final WmReportMapper wmReportMapper;
	private final SysUserService sysUserService;
	private final WmDaDistributionService wmDaDistributionService;
	private final WmDaPharmacyService wmDaPharmacyService;
	private final WmDaHospitalService wmDaHospitalService;
	private final WmDeptTaskTypeConfigService wmDeptTaskTypeConfigService;
	private final SysDeptService sysDeptService;
	private final SysDeptRelationMapper sysDeptRelationMapper;
	private final RedisTemplate<String,String> redisTemplate;

	@Override
	public IPage<WmReport> getReportListPage(Page page, WmReport wmReport, String scoreId, String name){
		IPage<WmReport> reportListPage = wmReportMapper.getReportListPage(page, wmReport, scoreId, name);

		List<WmReport> records = reportListPage.getRecords();

		for (WmReport report : records){

			// word报告状态
			WmReportOpt opt = new WmReportOpt();

			// 获取生成记录
			String key = String.format(CacheConstants.EXPORT_NB_REPORT_CACHE, report.getId());
			String o = redisTemplate.opsForValue().get(key);

			if (StrUtil.isNotEmpty(o)) {

				// 生成中的
				if ("GENERATING".equals(o)) {
					opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
				}
				// 生成失败的
				else if (o.startsWith("ERROR")) {
					opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					opt.setErrorMsg(o.split(StrUtil.UNDERLINE)[1]);
				}
				// 生成失败的2
				else if (!o.startsWith("http")) {
					opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					opt.setErrorMsg(o);
				}
				// 成功的
				else {
					Long expire = redisTemplate.opsForValue().getOperations().getExpire(key, TimeUnit.SECONDS);
					opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
					opt.setTtl(LocalDateTime.now().plusSeconds(expire));
					// o = "https://pre.yaoyi.net/admin/sys-file/wmkj/兼职学术信息沟通专员业绩报告(2249)-20211116191527.doc";
					opt.setLatestUrl(o);
				}


			} else {
				opt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
			}
			report.setReportOpt(opt);


			// excel报告状态
			WmReportOpt excelPpt = new WmReportOpt();

			// 获取生成记录
			String excelKey = String.format(CacheConstants.EXCEL_EXPORT_NB_REPORT_CACHE, report.getId());
			String excelO = redisTemplate.opsForValue().get(excelKey);

			if (StrUtil.isNotEmpty(excelO)) {

				// 生成中的
				if ("GENERATING".equals(excelO)) {
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
				}
				// 生成失败的
				else if (excelO.startsWith("ERROR")) {
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					excelPpt.setErrorMsg(excelO.split(StrUtil.UNDERLINE)[1]);
				}
				// 生成失败的2
				else if (!excelO.startsWith("http")) {
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					excelPpt.setErrorMsg(excelO);
				}
				// 成功的
				else {
					Long expire = redisTemplate.opsForValue().getOperations().getExpire(excelKey, TimeUnit.SECONDS);
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
					excelPpt.setTtl(LocalDateTime.now().plusSeconds(expire));
					// o = "https://pre.yaoyi.net/admin/sys-file/wmkj/兼职学术信息沟通专员业绩报告(2249)-20211116191527.doc";
					excelPpt.setLatestUrl(excelO);
				}


			} else {
				excelPpt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
			}
			report.setExcelReportOpt(excelPpt);

		}

		return reportListPage;
	}


	/**
	 * 报告审核状态修改--提交或结单
	 * reportStatus 1:待提交，2:已提交待审批 3：已部分审批  4：已退回 5:已审批待结单 6:已结单
	 * @param wmReport 报告审核表
	 * @return R
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R updateReportStatus(WmReport wmReport) {
		HnqzUser user= SecurityUtils.getUser();
		//提交修改状态，更新时间
		LambdaUpdateWrapper<WmReport> updateWrapper=new LambdaUpdateWrapper<>();
		updateWrapper.set(WmReport::getUpdatePeople,user.getId());
		updateWrapper.set(WmReport::getUpdateTime, LocalDateTime.now());
		updateWrapper.set(WmReport::getReportStatus, wmReport.getReportStatus());
		updateWrapper.eq(WmReport::getId,wmReport.getId());
		boolean isupdate=this.update(updateWrapper);
		if(!isupdate){
			return R.failed("更新报告状态失败");

		}
		//报告状态更新之后，要同时更新任务的报告审核状态
		updateTaskStatusByReport(wmReport);

		return R.ok("成功");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public R del(WmReport wmReport)  {
		LambdaUpdateWrapper<WmReport> updateWrapper=new LambdaUpdateWrapper<>();
		updateWrapper.set(WmReport::getDelFlag, wmReport.getDelFlag());
		updateWrapper.eq(WmReport::getId,wmReport.getId());
		boolean isupdate=this.update(updateWrapper);
		if(!isupdate){
			return R.failed("更新报告状态失败");
		}
		wmReport.setReportStatus(null);
		updateTaskStatusByReport(wmReport);
		return R.ok("删除成功");
	}



	/**
	 * 报告状态更新之后，要同时更新任务的报告审核状态
	 * @param wmReport
	 */
	@Transactional(rollbackFor = Exception.class)
	public void updateTaskStatusByReport(WmReport wmReport){
		WmScorePackage scorePackage=wmScorePackageService.getById(wmReport.getScoreId());
		//typeid  0 药企  1一级代理商 2二级代理商 3全职学术推广员 4兼职学术推广员
		LambdaUpdateWrapper<WmTask> updateWrapper=new LambdaUpdateWrapper<>();

		if("0".equals(scorePackage.getTypeid())){//药企
			if("1".equals(wmReport.getDelFlag())){
				updateWrapper.set(WmTask::getReportDrugId,null);
				updateWrapper.set(WmTask::getReportDrugApprovalOpinion,null);
				updateWrapper.set(WmTask::getReportDrugApprovalInfo,null);
			}
			updateWrapper.set(WmTask::getReportDrugApprovalStatus,wmReport.getReportStatus());
			updateWrapper.eq(WmTask::getReportDrugId,wmReport.getId());
		}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
			if("1".equals(wmReport.getDelFlag())){
				updateWrapper.set(WmTask::getReportOneId,null);
				updateWrapper.set(WmTask::getReportOneApprovalOpinion,null);
				updateWrapper.set(WmTask::getReportOneApprovalInfo,null);
			}
			if(SecurityUtils.getRoles().contains(3)){//药企
				updateWrapper.set(WmTask::getReportDrugApprovalStatus,wmReport.getReportStatus());
			}
			updateWrapper.set(WmTask::getReportOneApprovalStatus,wmReport.getReportStatus());
			updateWrapper.eq(WmTask::getReportOneId,wmReport.getId());
		}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
			if("1".equals(wmReport.getDelFlag())){
				updateWrapper.set(WmTask::getReportSecondId,null);
				updateWrapper.set(WmTask::getReportSecondApprovalOpinion,null);
				updateWrapper.set(WmTask::getReportSecondApprovalInfo,null);
			}
			if(SecurityUtils.getRoles().contains(4)){// 一级CSO
				updateWrapper.set(WmTask::getReportOneApprovalStatus,wmReport.getReportStatus());
			}
			updateWrapper.set(WmTask::getReportSecondApprovalStatus,wmReport.getReportStatus());
			updateWrapper.eq(WmTask::getReportSecondId,wmReport.getId());
		}
		wmTaskService.update(updateWrapper);
	}


	@Override
	@Transactional(rollbackFor = Exception.class)
	public R approvalReport(WmReport wmReport, List<String> taskIds, String approvalOpinion, String approvalInfo) {
		List<WmTask> taskList=new ArrayList<>();//该报告关联的所有未提交的任务

		WmScorePackage scorePackage = wmScorePackageService.getById(wmReport.getScoreId());
		// 如果是源头积分包，则判断是否审核入库调查信息
		if (StringUtils.isBlank(scorePackage.getRelationScoreId())) {
			List<WmTask> tasks = wmTaskService.list(Wrappers.<WmTask>lambdaQuery().in(WmTask::getId, taskIds));
			// 调查信息审核（15、30-医院 16-商业公司 18-药店）
			tasks.forEach(task -> {
				if ("15".equals(task.getTaskTypeId()) || "30".equals(task.getTaskTypeId())) {
					Map<String, Object> data = (Map<String, Object>) wmDaHospitalService.approvalHospital(task.getId()).getData();
					WmTask _wmTask = (WmTask) data.get("wmTask");
					Map<String, String> paramMap = BeanUtil.toBean(data.get("newinfo"), Map.class);
					paramMap.put("approvalInfo", approvalInfo);
					paramMap.put("approvalOpinion", approvalOpinion);
					paramMap.put("compareResult", _wmTask.getCompareResult());
					paramMap.put("taskId", task.getId());
					wmDaHospitalService.updateByApproval(paramMap);
				} else if ("16".equals(task.getTaskTypeId())) {
					Map<String, Object> data = (Map<String, Object>) wmDaDistributionService.approvalDistribution(task.getId()).getData();
					WmTask _wmTask = (WmTask) data.get("wmTask");
					Map<String, String> paramMap = BeanUtil.toBean(data.get("newinfo"), Map.class);
					paramMap.put("approvalInfo", approvalInfo);
					paramMap.put("approvalOpinion", approvalOpinion);
					paramMap.put("compareResult", _wmTask.getCompareResult());
					paramMap.put("taskId", task.getId());
					wmDaDistributionService.updateByApproval(paramMap);
				} else if ("18".equals(task.getTaskTypeId())) {
					Map<String, Object> data = (Map<String, Object>) wmDaPharmacyService.approvalPharmacy(task.getId()).getData();
					WmTask _wmTask = (WmTask) data.get("_wmTask");
					WmDaPharmacy info = (WmDaPharmacy) data.get("info");
					Map<String, String> paramMap = MapUtil.newHashMap(4);
					paramMap.put("infoId", info.getId());
					paramMap.put("approvalInfo", approvalInfo);
					paramMap.put("approvalOpinion", approvalOpinion);
					paramMap.put("compareResult", _wmTask.getCompareResult());
					paramMap.put("taskId", task.getId());
					wmDaPharmacyService.updateByapproval(paramMap);
				}
			});
		}

		//更新任务表审批意见,查询所有未审核的任务，判断报告关联任务是否全部审核
		LambdaUpdateWrapper<WmTask> updateTaskWrapper=new LambdaUpdateWrapper<>();
		WmTask wmTask=new WmTask();
		if(SecurityUtils.getRoles().contains(3)){//药企
			updateTaskWrapper.set(WmTask::getReportDrugApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
			updateTaskWrapper.set(WmTask::getReportDrugApprovalOpinion,approvalOpinion);
			updateTaskWrapper.set(WmTask::getReportDrugApprovalInfo,approvalInfo);

			if("1".equals(scorePackage.getTypeid())){
				updateTaskWrapper.set(WmTask::getReportOneApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion,approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportOneApprovalInfo,approvalInfo);
			}
			updateTaskWrapper.in(WmTask::getId,taskIds);

		}else if(SecurityUtils.getRoles().contains(4)){//一级代理商类型

			updateTaskWrapper.set(WmTask::getReportOneApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
			updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion,approvalOpinion);
			updateTaskWrapper.set(WmTask::getReportOneApprovalInfo,approvalInfo);
			if("2".equals(scorePackage.getTypeid())){
				updateTaskWrapper.set(WmTask::getReportSecondApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportSecondApprovalOpinion,approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportSecondApprovalInfo,approvalInfo);
			}
			updateTaskWrapper.in(WmTask::getId,taskIds);
		}
		if("0".equals(scorePackage.getTypeid())){//药企
			wmTask.setReportDrugId(wmReport.getId());
			wmTask.setReportDrugApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
			wmTask.setReportOneId(wmReport.getId());
			wmTask.setReportOneApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}else if("2".equals(scorePackage.getTypeid())) {//二级代理商类型
			wmTask.setReportSecondId(wmReport.getId());
			wmTask.setReportSecondApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}
//
		if ("1".equals(approvalOpinion)) {
			updateTaskWrapper.set(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val());
		} else if ("2".equals(approvalOpinion)) {
			// 审核不通过，则设置TaskStatus为不通过
			updateTaskWrapper.set(WmTask::getTaskStatus, TaskStatusEnum.FAIL_APPROVED.val());
		}
		updateTaskWrapper.set(WmTask::getRealFlag, "0");
		wmTaskService.update(updateTaskWrapper);
		taskList = wmTaskService.list(Wrappers.query(wmTask));
		//更新报告状态
		HnqzUser user= SecurityUtils.getUser();
		//提交修改状态，更新时间
		LambdaUpdateWrapper<WmReport> updateWrapper=new LambdaUpdateWrapper<>();

		//判断任务是不是已经全部审核完成，所有的任务都完成才算完成
		if(taskList.isEmpty()){//taskList为空，说明任务都已审核
			updateWrapper.set(WmReport::getReportStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
		}else {//不为空，说明是部分审批
			updateWrapper.set(WmReport::getReportStatus, ReportEnum.REPORT_STATUS_PART_APPROVAL.getType());
		}
		updateWrapper.set(WmReport::getUpdatePeople, user.getId());
		updateWrapper.set(WmReport::getUpdateTime, LocalDateTime.now());
		updateWrapper.eq(WmReport::getId, wmReport.getId());
		this.update(updateWrapper);

		LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.in(WmTask::getId, taskIds);
		List<WmTask> wmTaskList = wmTaskService.list(queryWrapper);
		if ("2".equals(approvalOpinion)) {
			// 审核不通过，则释放积分值，从而直接可以重新做任务
			wmTaskList.forEach(task -> {
				// 查询积分包状态
				WmScorePackageStatus scorePackageStatus = new WmScorePackageStatus();
				scorePackageStatus.setPackageId(task.getScorePackageId());
				scorePackageStatus.setUserId(task.getTaskUserId());
				scorePackageStatus.setStatus("2");//接单审核通过
				WmScorePackageStatus updateScorePackageStatus = wmScorePackageStatusService.getOne(Wrappers.query(scorePackageStatus));
				// 查询已提交的任务（除去审核不通过的任务）
				Wrapper<WmTask> taskQueryWrapper = Wrappers.<WmTask>lambdaQuery()
						.eq(WmTask::getScorePackageId, task.getScorePackageId())
						.in(WmTask::getTaskStatus, "1", "2", "3")
						.eq(WmTask::getRealFlag, "0");
				List<WmTask> submitedTaskList = wmTaskService.list(taskQueryWrapper);
				// 统计判断任务累计积分是否达到积分包的预设值
				int totalTaskScore = submitedTaskList.stream().mapToInt(WmTask::getScore).sum();
				WmScorePackage taskScorePackage = wmScorePackageService.getById(task.getScorePackageId());
				WmScorePackage updateScorePackage = new WmScorePackage();
				updateScorePackage.setId(task.getScorePackageId());
				if (totalTaskScore >= taskScorePackage.getScore()) {
					updateScorePackage.setScorePackageStatus(ScorePackageStatusEnum.TO_BE_APPROVAL.val());
					updateScorePackage.setTaskAddFlag("0");
					updateScorePackageStatus.setTaskAddFlag("0");
				}else {
					updateScorePackage.setScorePackageStatus(ScorePackageStatusEnum.IN_PROGRESS.val());
					updateScorePackage.setTaskAddFlag("1");
					updateScorePackageStatus.setTaskAddFlag("1");
				}
				wmScorePackageStatusService.updateById(updateScorePackageStatus);
				wmScorePackageService.updateById(updateScorePackage);
			});
		} else if ("1".equals(approvalOpinion)) {
			// 审核通过，则判断企业审核通过积分值是否达到积分包的预设值
			wmTaskList.forEach(task -> {
				Wrapper<WmTask> apporvalWrapper = Wrappers.<WmTask>lambdaQuery()
						.eq(WmTask::getScorePackageId, task.getScorePackageId())
						.eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
						.eq(WmTask::getRealFlag, "0")
						.and(wrapper -> wrapper.eq(WmTask::getReportDrugApprovalOpinion, "1")
								.or().eq(WmTask::getReportOneApprovalOpinion, "1"));
				List<WmTask> approvalTaskList = wmTaskService.list(apporvalWrapper);
				int totalScore = approvalTaskList.stream().mapToInt(WmTask::getScore).sum();
				WmScorePackage taskScorePackage = wmScorePackageService.getById(task.getScorePackageId());
				if (totalScore >= taskScorePackage.getScore()) {
					WmScorePackage updateScorePackage = new WmScorePackage();
					updateScorePackage.setId(task.getScorePackageId());
					updateScorePackage.setScorePackageStatus(ScorePackageStatusEnum.TO_BE_SETTLED.val());
					updateScorePackage.setTaskAddFlag("0");
					wmScorePackageService.updateById(updateScorePackage);
				}
			});
		}

		return R.ok("审批成功");
	}

	/**
	 * 企业根据任务类型审核任务
	 *
	 * @param taskIds
	 * @param approvalOpinion
	 * @param approvalInfo
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R<?> approvalReportByTaskType(List<String> taskIds, String approvalOpinion, String approvalInfo) {
		LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.in(WmTask::getId, taskIds);
		List<WmTask> wmTaskList = wmTaskService.list(queryWrapper);

		// 根据task任务表 反查report报告表
		// 以reportId（reportDrugId或reportOneId字段）为key进行分类
		Map<Integer, List<WmTask>> reportIdMap = wmTaskList.stream()
				.filter(wmTask -> wmTask.getReportDrugId() != null || wmTask.getReportOneId() != null)
				.collect(Collectors.groupingBy(wmTask -> wmTask.getReportDrugId() != null ?
						wmTask.getReportDrugId() : wmTask.getReportOneId()));

		AtomicBoolean successFlag = new AtomicBoolean(true);
		reportIdMap.forEach((reportId, wmTasks) -> {
			WmReport queryReport = this.getById(reportId);
			List<String> taskIdList = wmTasks.stream().map(WmTask::getId).collect(Collectors.toList());
			log.info("企业批量审核任务reportId={}, taskIds={}", reportId, taskIdList);

			// 调用原有审核方法
			R approvalReportResult = this.approvalReport(queryReport, taskIdList, approvalOpinion, approvalInfo);

			if (approvalReportResult.getCode() != 0) {
				successFlag.set(false);
			}
		});

		return successFlag.get() ? R.ok("审批成功") : R.failed("审批失败");
	}

	/**
	 * 报告管理--查看
	 * @param page
	 * @param wmReport
	 * @param isApprovalPage 1审批，0查看
	 * @return
	 */
	@Override
	public R getTaskListPage(Page page, WmReport wmReport, Integer isApprovalPage) {
		Page pages=null;
		WmScorePackage scorePackage=wmScorePackageService.getById(wmReport.getScoreId());
		WmTask wmTask=new WmTask();
		LambdaQueryWrapper<WmTask> lambdaQueryWrapper= Wrappers.<WmTask>query().lambda();
		lambdaQueryWrapper.and(wrapper -> wrapper.eq(WmTask::getReportDrugId, wmReport.getId())
				.or().eq(WmTask::getReportOneId, wmReport.getId())
				.or().eq(WmTask::getReportSecondId, wmReport.getId()));
		// 如果是审批页面，则过滤掉通过和不通过的数据（reportDrugApprovalOpinion或reportOneApprovalOpinion：1通过，2不通过）
		if (isApprovalPage != null && isApprovalPage.equals(1)) {
			if ("0".equals(scorePackage.getTypeid())) {//药企
				lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
			} else if ("1".equals(scorePackage.getTypeid())) {//一级代理商类型
				lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
			} else if ("2".equals(scorePackage.getTypeid())) {//二级代理商类型
				lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportOneApprovalOpinion));
			}
		}
		pages=wmTaskService.page(page, lambdaQueryWrapper);
//		if("0".equals(scorePackage.getTypeid())){//药企
//			wmTask.setReportDrugId(wmReport.getId());
//			pages=wmTaskService.page(page, Wrappers.query(wmTask));
//		}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
//			wmTask.setReportOneId(wmReport.getId());
//			pages=wmTaskService.page(page, Wrappers.query(wmTask));
//		}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
//			wmTask.setReportSecondId(wmReport.getId());
//			pages=wmTaskService.page(page, Wrappers.query(wmTask));
//		}
		if(pages==null){
			return R.ok();
		}
		List<WmTask> taskList=pages.getRecords();
		if(!taskList.isEmpty()){
			List<Map> resList =taskList.stream().map(wmtask -> {
				Map map = JSON.parseObject(JSON.toJSONString(wmtask), Map.class);
				map.put("reportNo",wmReport.getReportNo());
				map.put("scoreId",wmReport.getScoreId());
				if("0".equals(scorePackage.getTypeid())){//药企
					map.put("reportApprovalOpinion",wmtask.getReportDrugApprovalOpinion());
				}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
//					map.put("reportApprovalOpinion",wmtask.getReportOneApprovalOpinion());
					map.put("reportApprovalOpinion",wmtask.getReportDrugApprovalOpinion());
				}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
//					map.put("reportApprovalOpinion",wmtask.getReportSecondApprovalOpinion());
					map.put("reportApprovalOpinion",wmtask.getReportOneApprovalOpinion());
				}
				// 姓名
				if (StringUtils.isNumeric(wmtask.getTaskUserId())) {
					SysUser user = sysUserService.getById(Integer.parseInt(wmtask.getTaskUserId()));
					if (user != null) {
						map.put("taskUsername", user.getRealname());
					}
				}
				return map;
			}).collect(Collectors.toList());

			page.setRecords(resList);
		}
		return R.ok(pages);
	}

	/**
	 * 根据任务类型查询任务
	 *
	 * @param page 分页参数
	 * @param taskTypeName 任务类型名称
	 * @param entTaskStatus 审核状态 0待审核 1通过，2不通过
	 * @return
	 */
	@Override
	public R<?> taskPageByTaskType(Page<WmTaskVO> page, String taskTypeName, String[] entTaskStatus) {

		Integer deptId = SecurityUtils.getUser().getDeptId();

		LambdaQueryWrapper<WmTaskType> queryWrapper = Wrappers.<WmTaskType>lambdaQuery()
				.eq(WmTaskType::getTaskTypeLevel, "1")
				.eq(WmTaskType::getEnableFlag, EnableEnum.ENABLE.val());
		if (StrUtil.isNotBlank(taskTypeName)) {
			queryWrapper.eq(WmTaskType::getTaskTypeName, taskTypeName);
		} else {
			// 查询该企业可查询的任务类型权限
			// 本级dept对应任务类型配置
			WmDeptTaskTypeConfig deptTaskTypeConfig = wmDeptTaskTypeConfigService.getOne(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
					.eq(WmDeptTaskTypeConfig::getDeptId, deptId));
			List<WmDeptTaskTypeConfigJsonDTO> filterConfigJsonDTOList = null;
			if (deptTaskTypeConfig != null) {
				// 查询该企业当前任务类型下 的任务类型配置数据
				List<WmDeptTaskTypeConfigJsonDTO> configJsonDTOList =
						JSON.parseArray(deptTaskTypeConfig.getTaskTypeJson(), WmDeptTaskTypeConfigJsonDTO.class);
				filterConfigJsonDTOList = configJsonDTOList.stream()
						.filter(configJsonDTO -> configJsonDTO.getQualifiedTask())
						.collect(Collectors.toList());
		}

			// 查询上级的deptId
			SysDeptRelation ancestorDeptRelation = sysDeptRelationMapper.selectOne(Wrappers.<SysDeptRelation>lambdaQuery()
					.eq(SysDeptRelation::getDescendant, deptId)
					.ne(SysDeptRelation::getAncestor, deptId)
					.ne(SysDeptRelation::getAncestor, 1));
			// 上级dept对应任务类型配置
			List<WmDeptTaskTypeConfigJsonDTO> filterAncestorConfigList = null;
			if (ancestorDeptRelation != null) {
				WmDeptTaskTypeConfig ancestorDeptTaskTypeConfig = wmDeptTaskTypeConfigService.getOne(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
						.eq(WmDeptTaskTypeConfig::getDeptId, ancestorDeptRelation.getAncestor()));
				if (ancestorDeptTaskTypeConfig != null) {
					List<WmDeptTaskTypeConfigJsonDTO> ancestorConfigJsonDTOList =
							JSON.parseArray(ancestorDeptTaskTypeConfig.getTaskTypeJson(), WmDeptTaskTypeConfigJsonDTO.class);
					// 转换为key value形式
					filterAncestorConfigList = ancestorConfigJsonDTOList.stream()
							.filter(configJsonDTO -> configJsonDTO.getQualifiedTask() && configJsonDTO.getAssociateLower())
							.collect(Collectors.toList());
				}
			}
			// 任务配置为空，则直接返回空page
			if (CollectionUtil.isEmpty(filterConfigJsonDTOList) && CollectionUtil.isEmpty(filterAncestorConfigList)) {
				log.warn("任务类型权限不存在");
				return R.failed("任务类型不存在");
			}

			Set<String> taskTypeNames = new HashSet<>();
			if (CollUtil.isNotEmpty(filterConfigJsonDTOList)) {
				taskTypeNames.addAll(filterConfigJsonDTOList.stream().map(WmDeptTaskTypeConfigJsonDTO::getTaskTypeName)
						.collect(Collectors.toList()));
			}
			if (CollUtil.isNotEmpty(filterAncestorConfigList)) {
				taskTypeNames.addAll(filterAncestorConfigList.stream().map(WmDeptTaskTypeConfigJsonDTO::getTaskTypeName)
						.collect(Collectors.toList()));
			}
			queryWrapper.in(WmTaskType::getTaskTypeName, taskTypeNames);
		}
		List<WmTaskType> baseTaskTypeList = wmTaskTypeService.list(queryWrapper);
		if (CollUtil.isEmpty(baseTaskTypeList)) {
			log.warn("任务类型不存在tastTypeName={}", baseTaskTypeList);
			return R.failed("任务类型不存在");
		}

		// 根据drugEntId查询WmReport
//		List<WmReport> wmReportList = wmReportMapper.selectList(Wrappers.<WmReport>lambdaQuery()
//				.eq(WmReport::getDrugEntId, SecurityUtils.getUser().getDeptId()));

//		WmReport queryReport = new WmReport();
//		queryReport.setDrugEntId(SecurityUtils.getUser().getDeptId());
//		queryReport.setTaskTypeIdList(baseTaskTypeList.stream().map(WmTaskType::getId).collect(Collectors.toList()));

		Map<String, Object> entTaskStatusMap = new HashMap<>();
		SysDept dept = sysDeptService.getById(deptId);
		entTaskStatusMap.put("drugEntId", deptId);
		entTaskStatusMap.put("taskTypeIdList", baseTaskTypeList.stream().map(WmTaskType::getId).collect(Collectors.toList()));
//		entTaskStatusMap.put("reportIdList", wmReportList.stream().map(WmReport::getId).collect(Collectors.toList()));
		// 用来判断是否为药企deptLevel=2
		entTaskStatusMap.put("deptLevel", dept.getLevel());

		// 拆分是否通过状态、待审核状态，用于sql优化~
		if (ArrayUtil.isNotEmpty(entTaskStatus)) {
			// 通过状态
			String approvedStatus = null;
			// 待审核状态
			Boolean waitFlag = false;
			if (ArrayUtil.containsAll(entTaskStatus, "1", "2")) {
				// 通过+不通过
				approvedStatus = "3";
			} else if (ArrayUtil.contains(entTaskStatus, "1")) {
				// 通过
				approvedStatus = "1";
			} else if (ArrayUtil.contains(entTaskStatus, "2")) {
				// 不通过
				approvedStatus = "2";
			}
			if (ArrayUtil.contains(entTaskStatus, "0")) {
				// 待审核
				waitFlag = true;
			}
			entTaskStatusMap.put("entTaskStatus", entTaskStatus);
			entTaskStatusMap.put("approvedStatus", approvedStatus);
			entTaskStatusMap.put("waitFlag", waitFlag);
//			queryReport.setEntTaskStatus(entTaskStatusMap);
		}
		IPage<WmTaskVO> pages = wmTaskMapper.entTaskPageByTaskType(page, entTaskStatusMap);

		Page<Map<String, Object>> mapPage = new Page<>(pages.getCurrent(), pages.getSize(), pages.getTotal());
		List<WmTaskVO> taskList = pages.getRecords();
		if (CollUtil.isNotEmpty(taskList)) {
			List<Map<String, Object>> resList = taskList.stream().map(wmtask -> {
				Map<String, Object> map = JSON.parseObject(JSON.toJSONString(wmtask), Map.class);
				map.put("reportNo", wmtask.getReportNo());
				map.put("scoreId", wmtask.getScoreId());
				if (dept.getLevel() == 2) {//药企
					map.put("reportApprovalOpinion", wmtask.getReportDrugApprovalOpinion());
				} else if (dept.getLevel() == 3) {//一级代理商类型
					map.put("reportApprovalOpinion", wmtask.getReportDrugApprovalOpinion());
				} else if (dept.getLevel() == 4) {//二级代理商类型
					map.put("reportApprovalOpinion", wmtask.getReportOneApprovalOpinion());
				}
				// 姓名
				if (StringUtils.isNumeric(wmtask.getTaskUserId())) {
					SysUser user = sysUserService.getById(Integer.parseInt(wmtask.getTaskUserId()));
					if (user != null) {
						map.put("taskUsername", user.getRealname());
					}
				}
				map.put("scorePackageName", wmtask.getScorePackageName());
				// 积分包名称取当前企业发的包包名
				WmScorePackage scorePackage = wmScorePackageService.getById(wmtask.getScorePackageId());
				// 判断此包是不是当前企业发的
				if (!deptId.equals(Integer.parseInt(scorePackage.getSendPackageDeptId())) && StrUtil.isNotBlank(scorePackage.getRelationScoreId())) {
					// 不是则继续查询父级包
					WmScorePackage pScorePackage = wmScorePackageService.getById(scorePackage.getRelationScoreId());
					map.put("scorePackageName", pScorePackage.getScorePackageName());
					if (!deptId.equals(Integer.parseInt(pScorePackage.getSendPackageDeptId())) && StrUtil.isNotBlank(pScorePackage.getRelationScoreId())) {
						// 不是则再继续查询父级包
						WmScorePackage ppScorePackage = wmScorePackageService.getById(pScorePackage.getRelationScoreId());
						map.put("scorePackageName", ppScorePackage.getScorePackageName());
					}
				}
				return map;
			}).collect(Collectors.toList());

			mapPage.setRecords(resList);
		}
		return R.ok(mapPage);
	}
}



